//==============================================================================
//  GridSplineInterpolateSums.h
//
//  Copyright (C) 2010-2019 Tobias Toll and Thomas Ullrich 
//
//  This file is part of Sartre. 
//
//  This program is free software: you can redistribute it and/or modify 
//  it under the terms of the GNU General Public License as published by 
//  the Free Software Foundation.   
//  This program is distributed in the hope that it will be useful, 
//  but without any warranty; without even the implied warranty of 
//  merchantability or fitness for a particular purpose. See the 
//  GNU General Public License for more details. 
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//  Author: Thomas Ullrich
//  Last update: 
//  $Date: 2019-03-08 14:12:33 -0500 (Fri, 08 Mar 2019) $
//  $Author: ullrich $
//==============================================================================
//  
//  Sums that are performed for interpolations, written explicitely  
//  using these expanded forms will always result in a faster code  
//  
//  This algorithm was written by Cristian Lalescu.  
//  See  http://arxiv.org/abs/0905.3564 for details.  
//  The code here is slightly modified for use in Sartre.  
//  
//==============================================================================  
#ifndef GridSplineInterpolateSums_h  
#define GridSplineInterpolateSums_h  
       
#define SUM_1D_Q4_unsafe_macro(f, bx) \
                ((f(-1)*bx(-1) + f( 0)*bx( 0))+(f( 1)*bx( 1) + f( 2)*bx( 2)))  
       
#define SUM_2D_Q4_unsafe_macro(f, bx, by) \
                ((((f(-1,-1)*bx(-1) + f( 0,-1)*bx( 0))+(f( 1,-1)*bx( 1) + f( 2,-1)*bx( 2)))*by(-1) \
                + ((f(-1, 0)*bx(-1) + f( 0, 0)*bx( 0))+(f( 1, 0)*bx( 1) + f( 2, 0)*bx( 2)))*by( 0))\
                +(((f(-1, 1)*bx(-1) + f( 0, 1)*bx( 0))+(f( 1, 1)*bx( 1) + f( 2, 1)*bx( 2)))*by( 1) \
                + ((f(-1, 2)*bx(-1) + f( 0, 2)*bx( 0))+(f( 1, 2)*bx( 1) + f( 2, 2)*bx( 2)))*by( 2)))  
       
#define SUM_3D_Q4_unsafe_macro(f, bx, by, bz) \
                ((((((f(-1,-1,-1)*bx(-1) + f( 0,-1,-1)*bx( 0))+(f( 1,-1,-1)*bx( 1) + f( 2,-1,-1)*bx( 2)))*by(-1)          \
                  + ((f(-1, 0,-1)*bx(-1) + f( 0, 0,-1)*bx( 0))+(f( 1, 0,-1)*bx( 1) + f( 2, 0,-1)*bx( 2)))*by( 0))         \
                  +(((f(-1, 1,-1)*bx(-1) + f( 0, 1,-1)*bx( 0))+(f( 1, 1,-1)*bx( 1) + f( 2, 1,-1)*bx( 2)))*by( 1)          \
                  + ((f(-1, 2,-1)*bx(-1) + f( 0, 2,-1)*bx( 0))+(f( 1, 2,-1)*bx( 1) + f( 2, 2,-1)*bx( 2)))*by( 2)))*bz(-1) \
                + ((((f(-1,-1, 0)*bx(-1) + f( 0,-1, 0)*bx( 0))+(f( 1,-1, 0)*bx( 1) + f( 2,-1, 0)*bx( 2)))*by(-1)          \
                  + ((f(-1, 0, 0)*bx(-1) + f( 0, 0, 0)*bx( 0))+(f( 1, 0, 0)*bx( 1) + f( 2, 0, 0)*bx( 2)))*by( 0))         \
                  +(((f(-1, 1, 0)*bx(-1) + f( 0, 1, 0)*bx( 0))+(f( 1, 1, 0)*bx( 1) + f( 2, 1, 0)*bx( 2)))*by( 1)          \
                  + ((f(-1, 2, 0)*bx(-1) + f( 0, 2, 0)*bx( 0))+(f( 1, 2, 0)*bx( 1) + f( 2, 2, 0)*bx( 2)))*by( 2)))*bz( 0))\
                +(((((f(-1,-1, 1)*bx(-1) + f( 0,-1, 1)*bx( 0))+(f( 1,-1, 1)*bx( 1) + f( 2,-1, 1)*bx( 2)))*by(-1)          \
                  + ((f(-1, 0, 1)*bx(-1) + f( 0, 0, 1)*bx( 0))+(f( 1, 0, 1)*bx( 1) + f( 2, 0, 1)*bx( 2)))*by( 0))         \
                  +(((f(-1, 1, 1)*bx(-1) + f( 0, 1, 1)*bx( 0))+(f( 1, 1, 1)*bx( 1) + f( 2, 1, 1)*bx( 2)))*by( 1)          \
                  + ((f(-1, 2, 1)*bx(-1) + f( 0, 2, 1)*bx( 0))+(f( 1, 2, 1)*bx( 1) + f( 2, 2, 1)*bx( 2)))*by( 2)))*bz( 1) \
                + ((((f(-1,-1, 2)*bx(-1) + f( 0,-1, 2)*bx( 0))+(f( 1,-1, 2)*bx( 1) + f( 2,-1, 2)*bx( 2)))*by(-1)          \
                  + ((f(-1, 0, 2)*bx(-1) + f( 0, 0, 2)*bx( 0))+(f( 1, 0, 2)*bx( 1) + f( 2, 0, 2)*bx( 2)))*by( 0))         \
                  +(((f(-1, 1, 2)*bx(-1) + f( 0, 1, 2)*bx( 0))+(f( 1, 1, 2)*bx( 1) + f( 2, 1, 2)*bx( 2)))*by( 1)          \
                  + ((f(-1, 2, 2)*bx(-1) + f( 0, 2, 2)*bx( 0))+(f( 1, 2, 2)*bx( 1) + f( 2, 2, 2)*bx( 2)))*by( 2)))*bz( 2)))  
       
#define SUM_1D_Q6_unsafe_macro(f, bx) \
                (((f(-2)*bx(-2) + f(-1)*bx(-1))+ f( 0)*bx( 0))+(f( 1)*bx( 1) +(f( 2)*bx( 2) + f( 3)*bx( 3))))  
       
#define SUM_2D_Q6_unsafe_macro(f, bx, by) \
                ((((((f(-2,-2)*bx(-2) + f(-1,-2)*bx(-1))+ f( 0,-2)*bx( 0))+(f( 1,-2)*bx( 1) +(f( 2,-2)*bx( 2) + f( 3,-2)*bx( 3))))*by(-2)  \
                +  (((f(-2,-1)*bx(-2) + f(-1,-1)*bx(-1))+ f( 0,-1)*bx( 0))+(f( 1,-1)*bx( 1) +(f( 2,-1)*bx( 2) + f( 3,-1)*bx( 3))))*by(-1)) \
                +  (((f(-2, 0)*bx(-2) + f(-1, 0)*bx(-1))+ f( 0, 0)*bx( 0))+(f( 1, 0)*bx( 1) +(f( 2, 0)*bx( 2) + f( 3, 0)*bx( 3))))*by( 0)) \
                + ((((f(-2, 1)*bx(-2) + f(-1, 1)*bx(-1))+ f( 0, 1)*bx( 0))+(f( 1, 1)*bx( 1) +(f( 2, 1)*bx( 2) + f( 3, 1)*bx( 3))))*by( 1)  \
                + ((((f(-2, 2)*bx(-2) + f(-1, 2)*bx(-1))+ f( 0, 2)*bx( 0))+(f( 1, 2)*bx( 1) +(f( 2, 2)*bx( 2) + f( 3, 2)*bx( 3))))*by( 2)  \
                +  (((f(-2, 3)*bx(-2) + f(-1, 3)*bx(-1))+ f( 0, 3)*bx( 0))+(f( 1, 3)*bx( 1) +(f( 2, 3)*bx( 2) + f( 3, 3)*bx( 3))))*by( 3))))  
       
#define SUM_3D_Q6_unsafe_macro(f, bx, by, bz) \
                (((((((((f(-2,-2,-2)*bx(-2) + f(-1,-2,-2)*bx(-1))+ f( 0,-2,-2)*bx( 0))+(f( 1,-2,-2)*bx( 1) +(f( 2,-2,-2)*bx( 2) + f( 3,-2,-2)*bx( 3))))*by(-2)            \
                   +  (((f(-2,-1,-2)*bx(-2) + f(-1,-1,-2)*bx(-1))+ f( 0,-1,-2)*bx( 0))+(f( 1,-1,-2)*bx( 1) +(f( 2,-1,-2)*bx( 2) + f( 3,-1,-2)*bx( 3))))*by(-1))           \
                   +  (((f(-2, 0,-2)*bx(-2) + f(-1, 0,-2)*bx(-1))+ f( 0, 0,-2)*bx( 0))+(f( 1, 0,-2)*bx( 1) +(f( 2, 0,-2)*bx( 2) + f( 3, 0,-2)*bx( 3))))*by( 0))           \
                   + ((((f(-2, 1,-2)*bx(-2) + f(-1, 1,-2)*bx(-1))+ f( 0, 1,-2)*bx( 0))+(f( 1, 1,-2)*bx( 1) +(f( 2, 1,-2)*bx( 2) + f( 3, 1,-2)*bx( 3))))*by( 1)            \
                   + ((((f(-2, 2,-2)*bx(-2) + f(-1, 2,-2)*bx(-1))+ f( 0, 2,-2)*bx( 0))+(f( 1, 2,-2)*bx( 1) +(f( 2, 2,-2)*bx( 2) + f( 3, 2,-2)*bx( 3))))*by( 2)            \
                   +  (((f(-2, 3,-2)*bx(-2) + f(-1, 3,-2)*bx(-1))+ f( 0, 3,-2)*bx( 0))+(f( 1, 3,-2)*bx( 1) +(f( 2, 3,-2)*bx( 2) + f( 3, 3,-2)*bx( 3))))*by( 3))))*bz(-2)  \
                +  ((((((f(-2,-2,-1)*bx(-2) + f(-1,-2,-1)*bx(-1))+ f( 0,-2,-1)*bx( 0))+(f( 1,-2,-1)*bx( 1) +(f( 2,-2,-1)*bx( 2) + f( 3,-2,-1)*bx( 3))))*by(-2)            \
                   +  (((f(-2,-1,-1)*bx(-2) + f(-1,-1,-1)*bx(-1))+ f( 0,-1,-1)*bx( 0))+(f( 1,-1,-1)*bx( 1) +(f( 2,-1,-1)*bx( 2) + f( 3,-1,-1)*bx( 3))))*by(-1))           \
                   +  (((f(-2, 0,-1)*bx(-2) + f(-1, 0,-1)*bx(-1))+ f( 0, 0,-1)*bx( 0))+(f( 1, 0,-1)*bx( 1) +(f( 2, 0,-1)*bx( 2) + f( 3, 0,-1)*bx( 3))))*by( 0))           \
                   + ((((f(-2, 1,-1)*bx(-2) + f(-1, 1,-1)*bx(-1))+ f( 0, 1,-1)*bx( 0))+(f( 1, 1,-1)*bx( 1) +(f( 2, 1,-1)*bx( 2) + f( 3, 1,-1)*bx( 3))))*by( 1)            \
                   + ((((f(-2, 2,-1)*bx(-2) + f(-1, 2,-1)*bx(-1))+ f( 0, 2,-1)*bx( 0))+(f( 1, 2,-1)*bx( 1) +(f( 2, 2,-1)*bx( 2) + f( 3, 2,-1)*bx( 3))))*by( 2)            \
                   +  (((f(-2, 3,-1)*bx(-2) + f(-1, 3,-1)*bx(-1))+ f( 0, 3,-1)*bx( 0))+(f( 1, 3,-1)*bx( 1) +(f( 2, 3,-1)*bx( 2) + f( 3, 3,-1)*bx( 3))))*by( 3))))*bz(-1)) \
                 + ((((((f(-2,-2, 0)*bx(-2) + f(-1,-2, 0)*bx(-1))+ f( 0,-2, 0)*bx( 0))+(f( 1,-2, 0)*bx( 1) +(f( 2,-2, 0)*bx( 2) + f( 3,-2, 0)*bx( 3))))*by(-2)            \
                   +  (((f(-2,-1, 0)*bx(-2) + f(-1,-1, 0)*bx(-1))+ f( 0,-1, 0)*bx( 0))+(f( 1,-1, 0)*bx( 1) +(f( 2,-1, 0)*bx( 2) + f( 3,-1, 0)*bx( 3))))*by(-1))           \
                   +  (((f(-2, 0, 0)*bx(-2) + f(-1, 0, 0)*bx(-1))+ f( 0, 0, 0)*bx( 0))+(f( 1, 0, 0)*bx( 1) +(f( 2, 0, 0)*bx( 2) + f( 3, 0, 0)*bx( 3))))*by( 0))           \
                   + ((((f(-2, 1, 0)*bx(-2) + f(-1, 1, 0)*bx(-1))+ f( 0, 1, 0)*bx( 0))+(f( 1, 1, 0)*bx( 1) +(f( 2, 1, 0)*bx( 2) + f( 3, 1, 0)*bx( 3))))*by( 1)            \
                   + ((((f(-2, 2, 0)*bx(-2) + f(-1, 2, 0)*bx(-1))+ f( 0, 2, 0)*bx( 0))+(f( 1, 2, 0)*bx( 1) +(f( 2, 2, 0)*bx( 2) + f( 3, 2, 0)*bx( 3))))*by( 2)            \
                   +  (((f(-2, 3, 0)*bx(-2) + f(-1, 3, 0)*bx(-1))+ f( 0, 3, 0)*bx( 0))+(f( 1, 3, 0)*bx( 1) +(f( 2, 3, 0)*bx( 2) + f( 3, 3, 0)*bx( 3))))*by( 3))))*bz( 0)) \
                 +(((((((f(-2,-2, 1)*bx(-2) + f(-1,-2, 1)*bx(-1))+ f( 0,-2, 1)*bx( 0))+(f( 1,-2, 1)*bx( 1) +(f( 2,-2, 1)*bx( 2) + f( 3,-2, 1)*bx( 3))))*by(-2)            \
                   +  (((f(-2,-1, 1)*bx(-2) + f(-1,-1, 1)*bx(-1))+ f( 0,-1, 1)*bx( 0))+(f( 1,-1, 1)*bx( 1) +(f( 2,-1, 1)*bx( 2) + f( 3,-1, 1)*bx( 3))))*by(-1))           \
                   +  (((f(-2, 0, 1)*bx(-2) + f(-1, 0, 1)*bx(-1))+ f( 0, 0, 1)*bx( 0))+(f( 1, 0, 1)*bx( 1) +(f( 2, 0, 1)*bx( 2) + f( 3, 0, 1)*bx( 3))))*by( 0))           \
                   + ((((f(-2, 1, 1)*bx(-2) + f(-1, 1, 1)*bx(-1))+ f( 0, 1, 1)*bx( 0))+(f( 1, 1, 1)*bx( 1) +(f( 2, 1, 1)*bx( 2) + f( 3, 1, 1)*bx( 3))))*by( 1)            \
                   + ((((f(-2, 2, 1)*bx(-2) + f(-1, 2, 1)*bx(-1))+ f( 0, 2, 1)*bx( 0))+(f( 1, 2, 1)*bx( 1) +(f( 2, 2, 1)*bx( 2) + f( 3, 2, 1)*bx( 3))))*by( 2)            \
                   +  (((f(-2, 3, 1)*bx(-2) + f(-1, 3, 1)*bx(-1))+ f( 0, 3, 1)*bx( 0))+(f( 1, 3, 1)*bx( 1) +(f( 2, 3, 1)*bx( 2) + f( 3, 3, 1)*bx( 3))))*by( 3))))*bz( 1)  \
                + (((((((f(-2,-2, 2)*bx(-2) + f(-1,-2, 2)*bx(-1))+ f( 0,-2, 2)*bx( 0))+(f( 1,-2, 2)*bx( 1) +(f( 2,-2, 2)*bx( 2) + f( 3,-2, 2)*bx( 3))))*by(-2)            \
                   +  (((f(-2,-1, 2)*bx(-2) + f(-1,-1, 2)*bx(-1))+ f( 0,-1, 2)*bx( 0))+(f( 1,-1, 2)*bx( 1) +(f( 2,-1, 2)*bx( 2) + f( 3,-1, 2)*bx( 3))))*by(-1))           \
                   +  (((f(-2, 0, 2)*bx(-2) + f(-1, 0, 2)*bx(-1))+ f( 0, 0, 2)*bx( 0))+(f( 1, 0, 2)*bx( 1) +(f( 2, 0, 2)*bx( 2) + f( 3, 0, 2)*bx( 3))))*by( 0))           \
                   + ((((f(-2, 1, 2)*bx(-2) + f(-1, 1, 2)*bx(-1))+ f( 0, 1, 2)*bx( 0))+(f( 1, 1, 2)*bx( 1) +(f( 2, 1, 2)*bx( 2) + f( 3, 1, 2)*bx( 3))))*by( 1)            \
                   + ((((f(-2, 2, 2)*bx(-2) + f(-1, 2, 2)*bx(-1))+ f( 0, 2, 2)*bx( 0))+(f( 1, 2, 2)*bx( 1) +(f( 2, 2, 2)*bx( 2) + f( 3, 2, 2)*bx( 3))))*by( 2)            \
                   +  (((f(-2, 3, 2)*bx(-2) + f(-1, 3, 2)*bx(-1))+ f( 0, 3, 2)*bx( 0))+(f( 1, 3, 2)*bx( 1) +(f( 2, 3, 2)*bx( 2) + f( 3, 3, 2)*bx( 3))))*by( 3))))*bz( 2)  \
                +  ((((((f(-2,-2, 3)*bx(-2) + f(-1,-2, 3)*bx(-1))+ f( 0,-2, 3)*bx( 0))+(f( 1,-2, 3)*bx( 1) +(f( 2,-2, 3)*bx( 2) + f( 3,-2, 3)*bx( 3))))*by(-2)            \
                   +  (((f(-2,-1, 3)*bx(-2) + f(-1,-1, 3)*bx(-1))+ f( 0,-1, 3)*bx( 0))+(f( 1,-1, 3)*bx( 1) +(f( 2,-1, 3)*bx( 2) + f( 3,-1, 3)*bx( 3))))*by(-1))           \
                   +  (((f(-2, 0, 3)*bx(-2) + f(-1, 0, 3)*bx(-1))+ f( 0, 0, 3)*bx( 0))+(f( 1, 0, 3)*bx( 1) +(f( 2, 0, 3)*bx( 2) + f( 3, 0, 3)*bx( 3))))*by( 0))           \
                   + ((((f(-2, 1, 3)*bx(-2) + f(-1, 1, 3)*bx(-1))+ f( 0, 1, 3)*bx( 0))+(f( 1, 1, 3)*bx( 1) +(f( 2, 1, 3)*bx( 2) + f( 3, 1, 3)*bx( 3))))*by( 1)            \
                   + ((((f(-2, 2, 3)*bx(-2) + f(-1, 2, 3)*bx(-1))+ f( 0, 2, 3)*bx( 0))+(f( 1, 2, 3)*bx( 1) +(f( 2, 2, 3)*bx( 2) + f( 3, 2, 3)*bx( 3))))*by( 2)            \
                   +  (((f(-2, 3, 3)*bx(-2) + f(-1, 3, 3)*bx(-1))+ f( 0, 3, 3)*bx( 0))+(f( 1, 3, 3)*bx( 1) +(f( 2, 3, 3)*bx( 2) + f( 3, 3, 3)*bx( 3))))*by( 3))))*bz( 3))))  
       
#define SUM_1D_Q8_unsafe_macro(f, bx) \
                ((((f(-3)*bx(-3) + f(-2)*bx(-2))+ f(-1)*bx(-1))+ f( 0)*bx( 0))+(f( 1)*bx( 1) +(f( 2)*bx( 2) +(f( 3)*bx( 3) + f( 4)*bx( 4)))))  
       
#define SUM_2D_Q8_unsafe_macro(f, bx, by) \
                ((((((((f(-3,-3)*bx(-3) + f(-2,-3)*bx(-2))+ f(-1,-3)*bx(-1))+ f( 0,-3)*bx( 0))+(f( 1,-3)*bx( 1) +(f( 2,-3)*bx( 2) +(f( 3,-3)*bx( 3) + f( 4,-3)*bx( 4)))))*by(-3)   \
                   +((((f(-3,-2)*bx(-3) + f(-2,-2)*bx(-2))+ f(-1,-2)*bx(-1))+ f( 0,-2)*bx( 0))+(f( 1,-2)*bx( 1) +(f( 2,-2)*bx( 2) +(f( 3,-2)*bx( 3) + f( 4,-2)*bx( 4)))))*by(-2))  \
                   +((((f(-3,-1)*bx(-3) + f(-2,-1)*bx(-2))+ f(-1,-1)*bx(-1))+ f( 0,-1)*bx( 0))+(f( 1,-1)*bx( 1) +(f( 2,-1)*bx( 2) +(f( 3,-1)*bx( 3) + f( 4,-1)*bx( 4)))))*by(-1))  \
                   +((((f(-3, 0)*bx(-3) + f(-2, 0)*bx(-2))+ f(-1, 0)*bx(-1))+ f( 0, 0)*bx( 0))+(f( 1, 0)*bx( 1) +(f( 2, 0)*bx( 2) +(f( 3, 0)*bx( 3) + f( 4, 0)*bx( 4)))))*by( 0))  \
                  +(((((f(-3, 1)*bx(-3) + f(-2, 1)*bx(-2))+ f(-1, 1)*bx(-1))+ f( 0, 1)*bx( 0))+(f( 1, 1)*bx( 1) +(f( 2, 1)*bx( 2) +(f( 3, 1)*bx( 3) + f( 4, 1)*bx( 4)))))*by( 1)   \
                  +(((((f(-3, 2)*bx(-3) + f(-2, 2)*bx(-2))+ f(-1, 2)*bx(-1))+ f( 0, 2)*bx( 0))+(f( 1, 2)*bx( 1) +(f( 2, 2)*bx( 2) +(f( 3, 2)*bx( 3) + f( 4, 2)*bx( 4)))))*by( 2)   \
                  +(((((f(-3, 3)*bx(-3) + f(-2, 3)*bx(-2))+ f(-1, 3)*bx(-1))+ f( 0, 3)*bx( 0))+(f( 1, 3)*bx( 1) +(f( 2, 3)*bx( 2) +(f( 3, 3)*bx( 3) + f( 4, 3)*bx( 4)))))*by( 3)   \
                   +((((f(-3, 4)*bx(-3) + f(-2, 4)*bx(-2))+ f(-1, 4)*bx(-1))+ f( 0, 4)*bx( 0))+(f( 1, 4)*bx( 1) +(f( 2, 4)*bx( 2) +(f( 3, 4)*bx( 3) + f( 4, 4)*bx( 4)))))*by( 4)))))  
       
#define SUM_3D_Q8_unsafe_macro(f, bx, by, bz) \
                ((((((((((((f(-3,-3,-3)*bx(-3) + f(-2,-3,-3)*bx(-2))+ f(-1,-3,-3)*bx(-1))+ f( 0,-3,-3)*bx( 0))+(f( 1,-3,-3)*bx( 1) +(f( 2,-3,-3)*bx( 2) +(f( 3,-3,-3)*bx( 3) + f( 4,-3,-3)*bx( 4)))))*by(-3)              \
                       +((((f(-3,-2,-3)*bx(-3) + f(-2,-2,-3)*bx(-2))+ f(-1,-2,-3)*bx(-1))+ f( 0,-2,-3)*bx( 0))+(f( 1,-2,-3)*bx( 1) +(f( 2,-2,-3)*bx( 2) +(f( 3,-2,-3)*bx( 3) + f( 4,-2,-3)*bx( 4)))))*by(-2))             \
                       +((((f(-3,-1,-3)*bx(-3) + f(-2,-1,-3)*bx(-2))+ f(-1,-1,-3)*bx(-1))+ f( 0,-1,-3)*bx( 0))+(f( 1,-1,-3)*bx( 1) +(f( 2,-1,-3)*bx( 2) +(f( 3,-1,-3)*bx( 3) + f( 4,-1,-3)*bx( 4)))))*by(-1))             \
                       +((((f(-3, 0,-3)*bx(-3) + f(-2, 0,-3)*bx(-2))+ f(-1, 0,-3)*bx(-1))+ f( 0, 0,-3)*bx( 0))+(f( 1, 0,-3)*bx( 1) +(f( 2, 0,-3)*bx( 2) +(f( 3, 0,-3)*bx( 3) + f( 4, 0,-3)*bx( 4)))))*by( 0))             \
                      +(((((f(-3, 1,-3)*bx(-3) + f(-2, 1,-3)*bx(-2))+ f(-1, 1,-3)*bx(-1))+ f( 0, 1,-3)*bx( 0))+(f( 1, 1,-3)*bx( 1) +(f( 2, 1,-3)*bx( 2) +(f( 3, 1,-3)*bx( 3) + f( 4, 1,-3)*bx( 4)))))*by( 1)              \
                      +(((((f(-3, 2,-3)*bx(-3) + f(-2, 2,-3)*bx(-2))+ f(-1, 2,-3)*bx(-1))+ f( 0, 2,-3)*bx( 0))+(f( 1, 2,-3)*bx( 1) +(f( 2, 2,-3)*bx( 2) +(f( 3, 2,-3)*bx( 3) + f( 4, 2,-3)*bx( 4)))))*by( 2)              \
                      +(((((f(-3, 3,-3)*bx(-3) + f(-2, 3,-3)*bx(-2))+ f(-1, 3,-3)*bx(-1))+ f( 0, 3,-3)*bx( 0))+(f( 1, 3,-3)*bx( 1) +(f( 2, 3,-3)*bx( 2) +(f( 3, 3,-3)*bx( 3) + f( 4, 3,-3)*bx( 4)))))*by( 3)              \
                       +((((f(-3, 4,-3)*bx(-3) + f(-2, 4,-3)*bx(-2))+ f(-1, 4,-3)*bx(-1))+ f( 0, 4,-3)*bx( 0))+(f( 1, 4,-3)*bx( 1) +(f( 2, 4,-3)*bx( 2) +(f( 3, 4,-3)*bx( 3) + f( 4, 4,-3)*bx( 4)))))*by( 4)))))*bz(-3)   \
                   +((((((((f(-3,-3,-2)*bx(-3) + f(-2,-3,-2)*bx(-2))+ f(-1,-3,-2)*bx(-1))+ f( 0,-3,-2)*bx( 0))+(f( 1,-3,-2)*bx( 1) +(f( 2,-3,-2)*bx( 2) +(f( 3,-3,-2)*bx( 3) + f( 4,-3,-2)*bx( 4)))))*by(-3)              \
                       +((((f(-3,-2,-2)*bx(-3) + f(-2,-2,-2)*bx(-2))+ f(-1,-2,-2)*bx(-1))+ f( 0,-2,-2)*bx( 0))+(f( 1,-2,-2)*bx( 1) +(f( 2,-2,-2)*bx( 2) +(f( 3,-2,-2)*bx( 3) + f( 4,-2,-2)*bx( 4)))))*by(-2))             \
                       +((((f(-3,-1,-2)*bx(-3) + f(-2,-1,-2)*bx(-2))+ f(-1,-1,-2)*bx(-1))+ f( 0,-1,-2)*bx( 0))+(f( 1,-1,-2)*bx( 1) +(f( 2,-1,-2)*bx( 2) +(f( 3,-1,-2)*bx( 3) + f( 4,-1,-2)*bx( 4)))))*by(-1))             \
                       +((((f(-3, 0,-2)*bx(-3) + f(-2, 0,-2)*bx(-2))+ f(-1, 0,-2)*bx(-1))+ f( 0, 0,-2)*bx( 0))+(f( 1, 0,-2)*bx( 1) +(f( 2, 0,-2)*bx( 2) +(f( 3, 0,-2)*bx( 3) + f( 4, 0,-2)*bx( 4)))))*by( 0))             \
                      +(((((f(-3, 1,-2)*bx(-3) + f(-2, 1,-2)*bx(-2))+ f(-1, 1,-2)*bx(-1))+ f( 0, 1,-2)*bx( 0))+(f( 1, 1,-2)*bx( 1) +(f( 2, 1,-2)*bx( 2) +(f( 3, 1,-2)*bx( 3) + f( 4, 1,-2)*bx( 4)))))*by( 1)              \
                      +(((((f(-3, 2,-2)*bx(-3) + f(-2, 2,-2)*bx(-2))+ f(-1, 2,-2)*bx(-1))+ f( 0, 2,-2)*bx( 0))+(f( 1, 2,-2)*bx( 1) +(f( 2, 2,-2)*bx( 2) +(f( 3, 2,-2)*bx( 3) + f( 4, 2,-2)*bx( 4)))))*by( 2)              \
                      +(((((f(-3, 3,-2)*bx(-3) + f(-2, 3,-2)*bx(-2))+ f(-1, 3,-2)*bx(-1))+ f( 0, 3,-2)*bx( 0))+(f( 1, 3,-2)*bx( 1) +(f( 2, 3,-2)*bx( 2) +(f( 3, 3,-2)*bx( 3) + f( 4, 3,-2)*bx( 4)))))*by( 3)              \
                       +((((f(-3, 4,-2)*bx(-3) + f(-2, 4,-2)*bx(-2))+ f(-1, 4,-2)*bx(-1))+ f( 0, 4,-2)*bx( 0))+(f( 1, 4,-2)*bx( 1) +(f( 2, 4,-2)*bx( 2) +(f( 3, 4,-2)*bx( 3) + f( 4, 4,-2)*bx( 4)))))*by( 4)))))*bz(-2))  \
                   +((((((((f(-3,-3,-1)*bx(-3) + f(-2,-3,-1)*bx(-2))+ f(-1,-3,-1)*bx(-1))+ f( 0,-3,-1)*bx( 0))+(f( 1,-3,-1)*bx( 1) +(f( 2,-3,-1)*bx( 2) +(f( 3,-3,-1)*bx( 3) + f( 4,-3,-1)*bx( 4)))))*by(-3)              \
                       +((((f(-3,-2,-1)*bx(-3) + f(-2,-2,-1)*bx(-2))+ f(-1,-2,-1)*bx(-1))+ f( 0,-2,-1)*bx( 0))+(f( 1,-2,-1)*bx( 1) +(f( 2,-2,-1)*bx( 2) +(f( 3,-2,-1)*bx( 3) + f( 4,-2,-1)*bx( 4)))))*by(-2))             \
                       +((((f(-3,-1,-1)*bx(-3) + f(-2,-1,-1)*bx(-2))+ f(-1,-1,-1)*bx(-1))+ f( 0,-1,-1)*bx( 0))+(f( 1,-1,-1)*bx( 1) +(f( 2,-1,-1)*bx( 2) +(f( 3,-1,-1)*bx( 3) + f( 4,-1,-1)*bx( 4)))))*by(-1))             \
                       +((((f(-3, 0,-1)*bx(-3) + f(-2, 0,-1)*bx(-2))+ f(-1, 0,-1)*bx(-1))+ f( 0, 0,-1)*bx( 0))+(f( 1, 0,-1)*bx( 1) +(f( 2, 0,-1)*bx( 2) +(f( 3, 0,-1)*bx( 3) + f( 4, 0,-1)*bx( 4)))))*by( 0))             \
                      +(((((f(-3, 1,-1)*bx(-3) + f(-2, 1,-1)*bx(-2))+ f(-1, 1,-1)*bx(-1))+ f( 0, 1,-1)*bx( 0))+(f( 1, 1,-1)*bx( 1) +(f( 2, 1,-1)*bx( 2) +(f( 3, 1,-1)*bx( 3) + f( 4, 1,-1)*bx( 4)))))*by( 1)              \
                      +(((((f(-3, 2,-1)*bx(-3) + f(-2, 2,-1)*bx(-2))+ f(-1, 2,-1)*bx(-1))+ f( 0, 2,-1)*bx( 0))+(f( 1, 2,-1)*bx( 1) +(f( 2, 2,-1)*bx( 2) +(f( 3, 2,-1)*bx( 3) + f( 4, 2,-1)*bx( 4)))))*by( 2)              \
                      +(((((f(-3, 3,-1)*bx(-3) + f(-2, 3,-1)*bx(-2))+ f(-1, 3,-1)*bx(-1))+ f( 0, 3,-1)*bx( 0))+(f( 1, 3,-1)*bx( 1) +(f( 2, 3,-1)*bx( 2) +(f( 3, 3,-1)*bx( 3) + f( 4, 3,-1)*bx( 4)))))*by( 3)              \
                       +((((f(-3, 4,-1)*bx(-3) + f(-2, 4,-1)*bx(-2))+ f(-1, 4,-1)*bx(-1))+ f( 0, 4,-1)*bx( 0))+(f( 1, 4,-1)*bx( 1) +(f( 2, 4,-1)*bx( 2) +(f( 3, 4,-1)*bx( 3) + f( 4, 4,-1)*bx( 4)))))*by( 4)))))*bz(-1))  \
                   +((((((((f(-3,-3, 0)*bx(-3) + f(-2,-3, 0)*bx(-2))+ f(-1,-3, 0)*bx(-1))+ f( 0,-3, 0)*bx( 0))+(f( 1,-3, 0)*bx( 1) +(f( 2,-3, 0)*bx( 2) +(f( 3,-3, 0)*bx( 3) + f( 4,-3, 0)*bx( 4)))))*by(-3)              \
                       +((((f(-3,-2, 0)*bx(-3) + f(-2,-2, 0)*bx(-2))+ f(-1,-2, 0)*bx(-1))+ f( 0,-2, 0)*bx( 0))+(f( 1,-2, 0)*bx( 1) +(f( 2,-2, 0)*bx( 2) +(f( 3,-2, 0)*bx( 3) + f( 4,-2, 0)*bx( 4)))))*by(-2))             \
                       +((((f(-3,-1, 0)*bx(-3) + f(-2,-1, 0)*bx(-2))+ f(-1,-1, 0)*bx(-1))+ f( 0,-1, 0)*bx( 0))+(f( 1,-1, 0)*bx( 1) +(f( 2,-1, 0)*bx( 2) +(f( 3,-1, 0)*bx( 3) + f( 4,-1, 0)*bx( 4)))))*by(-1))             \
                       +((((f(-3, 0, 0)*bx(-3) + f(-2, 0, 0)*bx(-2))+ f(-1, 0, 0)*bx(-1))+ f( 0, 0, 0)*bx( 0))+(f( 1, 0, 0)*bx( 1) +(f( 2, 0, 0)*bx( 2) +(f( 3, 0, 0)*bx( 3) + f( 4, 0, 0)*bx( 4)))))*by( 0))             \
                      +(((((f(-3, 1, 0)*bx(-3) + f(-2, 1, 0)*bx(-2))+ f(-1, 1, 0)*bx(-1))+ f( 0, 1, 0)*bx( 0))+(f( 1, 1, 0)*bx( 1) +(f( 2, 1, 0)*bx( 2) +(f( 3, 1, 0)*bx( 3) + f( 4, 1, 0)*bx( 4)))))*by( 1)              \
                      +(((((f(-3, 2, 0)*bx(-3) + f(-2, 2, 0)*bx(-2))+ f(-1, 2, 0)*bx(-1))+ f( 0, 2, 0)*bx( 0))+(f( 1, 2, 0)*bx( 1) +(f( 2, 2, 0)*bx( 2) +(f( 3, 2, 0)*bx( 3) + f( 4, 2, 0)*bx( 4)))))*by( 2)              \
                      +(((((f(-3, 3, 0)*bx(-3) + f(-2, 3, 0)*bx(-2))+ f(-1, 3, 0)*bx(-1))+ f( 0, 3, 0)*bx( 0))+(f( 1, 3, 0)*bx( 1) +(f( 2, 3, 0)*bx( 2) +(f( 3, 3, 0)*bx( 3) + f( 4, 3, 0)*bx( 4)))))*by( 3)              \
                       +((((f(-3, 4, 0)*bx(-3) + f(-2, 4, 0)*bx(-2))+ f(-1, 4, 0)*bx(-1))+ f( 0, 4, 0)*bx( 0))+(f( 1, 4, 0)*bx( 1) +(f( 2, 4, 0)*bx( 2) +(f( 3, 4, 0)*bx( 3) + f( 4, 4, 0)*bx( 4)))))*by( 4)))))*bz( 0))  \
                +  (((((((((f(-3,-3, 1)*bx(-3) + f(-2,-3, 1)*bx(-2))+ f(-1,-3, 1)*bx(-1))+ f( 0,-3, 1)*bx( 0))+(f( 1,-3, 1)*bx( 1) +(f( 2,-3, 1)*bx( 2) +(f( 3,-3, 1)*bx( 3) + f( 4,-3, 1)*bx( 4)))))*by(-3)              \
                       +((((f(-3,-2, 1)*bx(-3) + f(-2,-2, 1)*bx(-2))+ f(-1,-2, 1)*bx(-1))+ f( 0,-2, 1)*bx( 0))+(f( 1,-2, 1)*bx( 1) +(f( 2,-2, 1)*bx( 2) +(f( 3,-2, 1)*bx( 3) + f( 4,-2, 1)*bx( 4)))))*by(-2))             \
                       +((((f(-3,-1, 1)*bx(-3) + f(-2,-1, 1)*bx(-2))+ f(-1,-1, 1)*bx(-1))+ f( 0,-1, 1)*bx( 0))+(f( 1,-1, 1)*bx( 1) +(f( 2,-1, 1)*bx( 2) +(f( 3,-1, 1)*bx( 3) + f( 4,-1, 1)*bx( 4)))))*by(-1))             \
                       +((((f(-3, 0, 1)*bx(-3) + f(-2, 0, 1)*bx(-2))+ f(-1, 0, 1)*bx(-1))+ f( 0, 0, 1)*bx( 0))+(f( 1, 0, 1)*bx( 1) +(f( 2, 0, 1)*bx( 2) +(f( 3, 0, 1)*bx( 3) + f( 4, 0, 1)*bx( 4)))))*by( 0))             \
                      +(((((f(-3, 1, 1)*bx(-3) + f(-2, 1, 1)*bx(-2))+ f(-1, 1, 1)*bx(-1))+ f( 0, 1, 1)*bx( 0))+(f( 1, 1, 1)*bx( 1) +(f( 2, 1, 1)*bx( 2) +(f( 3, 1, 1)*bx( 3) + f( 4, 1, 1)*bx( 4)))))*by( 1)              \
                      +(((((f(-3, 2, 1)*bx(-3) + f(-2, 2, 1)*bx(-2))+ f(-1, 2, 1)*bx(-1))+ f( 0, 2, 1)*bx( 0))+(f( 1, 2, 1)*bx( 1) +(f( 2, 2, 1)*bx( 2) +(f( 3, 2, 1)*bx( 3) + f( 4, 2, 1)*bx( 4)))))*by( 2)              \
                      +(((((f(-3, 3, 1)*bx(-3) + f(-2, 3, 1)*bx(-2))+ f(-1, 3, 1)*bx(-1))+ f( 0, 3, 1)*bx( 0))+(f( 1, 3, 1)*bx( 1) +(f( 2, 3, 1)*bx( 2) +(f( 3, 3, 1)*bx( 3) + f( 4, 3, 1)*bx( 4)))))*by( 3)              \
                       +((((f(-3, 4, 1)*bx(-3) + f(-2, 4, 1)*bx(-2))+ f(-1, 4, 1)*bx(-1))+ f( 0, 4, 1)*bx( 0))+(f( 1, 4, 1)*bx( 1) +(f( 2, 4, 1)*bx( 2) +(f( 3, 4, 1)*bx( 3) + f( 4, 4, 1)*bx( 4)))))*by( 4)))))*bz( 1)   \
                  +(((((((((f(-3,-3, 2)*bx(-3) + f(-2,-3, 2)*bx(-2))+ f(-1,-3, 2)*bx(-1))+ f( 0,-3, 2)*bx( 0))+(f( 1,-3, 2)*bx( 1) +(f( 2,-3, 2)*bx( 2) +(f( 3,-3, 2)*bx( 3) + f( 4,-3, 2)*bx( 4)))))*by(-3)              \
                       +((((f(-3,-2, 2)*bx(-3) + f(-2,-2, 2)*bx(-2))+ f(-1,-2, 2)*bx(-1))+ f( 0,-2, 2)*bx( 0))+(f( 1,-2, 2)*bx( 1) +(f( 2,-2, 2)*bx( 2) +(f( 3,-2, 2)*bx( 3) + f( 4,-2, 2)*bx( 4)))))*by(-2))             \
                       +((((f(-3,-1, 2)*bx(-3) + f(-2,-1, 2)*bx(-2))+ f(-1,-1, 2)*bx(-1))+ f( 0,-1, 2)*bx( 0))+(f( 1,-1, 2)*bx( 1) +(f( 2,-1, 2)*bx( 2) +(f( 3,-1, 2)*bx( 3) + f( 4,-1, 2)*bx( 4)))))*by(-1))             \
                       +((((f(-3, 0, 2)*bx(-3) + f(-2, 0, 2)*bx(-2))+ f(-1, 0, 2)*bx(-1))+ f( 0, 0, 2)*bx( 0))+(f( 1, 0, 2)*bx( 1) +(f( 2, 0, 2)*bx( 2) +(f( 3, 0, 2)*bx( 3) + f( 4, 0, 2)*bx( 4)))))*by( 0))             \
                      +(((((f(-3, 1, 2)*bx(-3) + f(-2, 1, 2)*bx(-2))+ f(-1, 1, 2)*bx(-1))+ f( 0, 1, 2)*bx( 0))+(f( 1, 1, 2)*bx( 1) +(f( 2, 1, 2)*bx( 2) +(f( 3, 1, 2)*bx( 3) + f( 4, 1, 2)*bx( 4)))))*by( 1)              \
                      +(((((f(-3, 2, 2)*bx(-3) + f(-2, 2, 2)*bx(-2))+ f(-1, 2, 2)*bx(-1))+ f( 0, 2, 2)*bx( 0))+(f( 1, 2, 2)*bx( 1) +(f( 2, 2, 2)*bx( 2) +(f( 3, 2, 2)*bx( 3) + f( 4, 2, 2)*bx( 4)))))*by( 2)              \
                      +(((((f(-3, 3, 2)*bx(-3) + f(-2, 3, 2)*bx(-2))+ f(-1, 3, 2)*bx(-1))+ f( 0, 3, 2)*bx( 0))+(f( 1, 3, 2)*bx( 1) +(f( 2, 3, 2)*bx( 2) +(f( 3, 3, 2)*bx( 3) + f( 4, 3, 2)*bx( 4)))))*by( 3)              \
                       +((((f(-3, 4, 2)*bx(-3) + f(-2, 4, 2)*bx(-2))+ f(-1, 4, 2)*bx(-1))+ f( 0, 4, 2)*bx( 0))+(f( 1, 4, 2)*bx( 1) +(f( 2, 4, 2)*bx( 2) +(f( 3, 4, 2)*bx( 3) + f( 4, 4, 2)*bx( 4)))))*by( 4)))))*bz( 2)   \
                  +(((((((((f(-3,-3, 3)*bx(-3) + f(-2,-3, 3)*bx(-2))+ f(-1,-3, 3)*bx(-1))+ f( 0,-3, 3)*bx( 0))+(f( 1,-3, 3)*bx( 1) +(f( 2,-3, 3)*bx( 2) +(f( 3,-3, 3)*bx( 3) + f( 4,-3, 3)*bx( 4)))))*by(-3)              \
                       +((((f(-3,-2, 3)*bx(-3) + f(-2,-2, 3)*bx(-2))+ f(-1,-2, 3)*bx(-1))+ f( 0,-2, 3)*bx( 0))+(f( 1,-2, 3)*bx( 1) +(f( 2,-2, 3)*bx( 2) +(f( 3,-2, 3)*bx( 3) + f( 4,-2, 3)*bx( 4)))))*by(-2))             \
                       +((((f(-3,-1, 3)*bx(-3) + f(-2,-1, 3)*bx(-2))+ f(-1,-1, 3)*bx(-1))+ f( 0,-1, 3)*bx( 0))+(f( 1,-1, 3)*bx( 1) +(f( 2,-1, 3)*bx( 2) +(f( 3,-1, 3)*bx( 3) + f( 4,-1, 3)*bx( 4)))))*by(-1))             \
                       +((((f(-3, 0, 3)*bx(-3) + f(-2, 0, 3)*bx(-2))+ f(-1, 0, 3)*bx(-1))+ f( 0, 0, 3)*bx( 0))+(f( 1, 0, 3)*bx( 1) +(f( 2, 0, 3)*bx( 2) +(f( 3, 0, 3)*bx( 3) + f( 4, 0, 3)*bx( 4)))))*by( 0))             \
                      +(((((f(-3, 1, 3)*bx(-3) + f(-2, 1, 3)*bx(-2))+ f(-1, 1, 3)*bx(-1))+ f( 0, 1, 3)*bx( 0))+(f( 1, 1, 3)*bx( 1) +(f( 2, 1, 3)*bx( 2) +(f( 3, 1, 3)*bx( 3) + f( 4, 1, 3)*bx( 4)))))*by( 1)              \
                      +(((((f(-3, 2, 3)*bx(-3) + f(-2, 2, 3)*bx(-2))+ f(-1, 2, 3)*bx(-1))+ f( 0, 2, 3)*bx( 0))+(f( 1, 2, 3)*bx( 1) +(f( 2, 2, 3)*bx( 2) +(f( 3, 2, 3)*bx( 3) + f( 4, 2, 3)*bx( 4)))))*by( 2)              \
                      +(((((f(-3, 3, 3)*bx(-3) + f(-2, 3, 3)*bx(-2))+ f(-1, 3, 3)*bx(-1))+ f( 0, 3, 3)*bx( 0))+(f( 1, 3, 3)*bx( 1) +(f( 2, 3, 3)*bx( 2) +(f( 3, 3, 3)*bx( 3) + f( 4, 3, 3)*bx( 4)))))*by( 3)              \
                       +((((f(-3, 4, 3)*bx(-3) + f(-2, 4, 3)*bx(-2))+ f(-1, 4, 3)*bx(-1))+ f( 0, 4, 3)*bx( 0))+(f( 1, 4, 3)*bx( 1) +(f( 2, 4, 3)*bx( 2) +(f( 3, 4, 3)*bx( 3) + f( 4, 4, 3)*bx( 4)))))*by( 4)))))*bz( 3)   \
                   +((((((((f(-3,-3, 4)*bx(-3) + f(-2,-3, 4)*bx(-2))+ f(-1,-3, 4)*bx(-1))+ f( 0,-3, 4)*bx( 0))+(f( 1,-3, 4)*bx( 1) +(f( 2,-3, 4)*bx( 2) +(f( 3,-3, 4)*bx( 3) + f( 4,-3, 4)*bx( 4)))))*by(-3)              \
                       +((((f(-3,-2, 4)*bx(-3) + f(-2,-2, 4)*bx(-2))+ f(-1,-2, 4)*bx(-1))+ f( 0,-2, 4)*bx( 0))+(f( 1,-2, 4)*bx( 1) +(f( 2,-2, 4)*bx( 2) +(f( 3,-2, 4)*bx( 3) + f( 4,-2, 4)*bx( 4)))))*by(-2))             \
                       +((((f(-3,-1, 4)*bx(-3) + f(-2,-1, 4)*bx(-2))+ f(-1,-1, 4)*bx(-1))+ f( 0,-1, 4)*bx( 0))+(f( 1,-1, 4)*bx( 1) +(f( 2,-1, 4)*bx( 2) +(f( 3,-1, 4)*bx( 3) + f( 4,-1, 4)*bx( 4)))))*by(-1))             \
                       +((((f(-3, 0, 4)*bx(-3) + f(-2, 0, 4)*bx(-2))+ f(-1, 0, 4)*bx(-1))+ f( 0, 0, 4)*bx( 0))+(f( 1, 0, 4)*bx( 1) +(f( 2, 0, 4)*bx( 2) +(f( 3, 0, 4)*bx( 3) + f( 4, 0, 4)*bx( 4)))))*by( 0))             \
                      +(((((f(-3, 1, 4)*bx(-3) + f(-2, 1, 4)*bx(-2))+ f(-1, 1, 4)*bx(-1))+ f( 0, 1, 4)*bx( 0))+(f( 1, 1, 4)*bx( 1) +(f( 2, 1, 4)*bx( 2) +(f( 3, 1, 4)*bx( 3) + f( 4, 1, 4)*bx( 4)))))*by( 1)              \
                      +(((((f(-3, 2, 4)*bx(-3) + f(-2, 2, 4)*bx(-2))+ f(-1, 2, 4)*bx(-1))+ f( 0, 2, 4)*bx( 0))+(f( 1, 2, 4)*bx( 1) +(f( 2, 2, 4)*bx( 2) +(f( 3, 2, 4)*bx( 3) + f( 4, 2, 4)*bx( 4)))))*by( 2)              \
                      +(((((f(-3, 3, 4)*bx(-3) + f(-2, 3, 4)*bx(-2))+ f(-1, 3, 4)*bx(-1))+ f( 0, 3, 4)*bx( 0))+(f( 1, 3, 4)*bx( 1) +(f( 2, 3, 4)*bx( 2) +(f( 3, 3, 4)*bx( 3) + f( 4, 3, 4)*bx( 4)))))*by( 3)              \
                       +((((f(-3, 4, 4)*bx(-3) + f(-2, 4, 4)*bx(-2))+ f(-1, 4, 4)*bx(-1))+ f( 0, 4, 4)*bx( 0))+(f( 1, 4, 4)*bx( 1) +(f( 2, 4, 4)*bx( 2) +(f( 3, 4, 4)*bx( 3) + f( 4, 4, 4)*bx( 4)))))*by( 4)))))*bz( 4)))))  
       
#endif  
       
